package com.huasheng.quant.open.gateway.api;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.huasheng.quant.open.gateway.sdk.constant.ISystemError;
import com.huasheng.quant.open.gateway.sdk.constant.OpenApiConstant;
import com.huasheng.quant.open.gateway.sdk.constant.OpenApiRequestConstant;
import com.huasheng.quant.open.gateway.sdk.constant.RateType;
import com.huasheng.quant.open.gateway.sdk.constant.futures.FuturesValidTimeType;
import com.huasheng.quant.open.gateway.sdk.constant.trade.SessionType;
import com.huasheng.quant.open.gateway.sdk.constant.trade.TradeRequestConstant;
import com.huasheng.quant.open.gateway.sdk.domain.ModelResult;
import com.huasheng.quant.open.gateway.sdk.handlers.DefaultClientApplication;
import com.huasheng.quant.open.gateway.sdk.notify.NotifyMessageRunnable;
import com.huasheng.quant.open.gateway.sdk.vo.*;
import com.huasheng.quant.open.gateway.sdk.vo.algo.*;
import com.huasheng.quant.open.gateway.sdk.vo.futures.*;
import com.huasheng.quant.open.gateway.sdk.vo.hq.*;
import com.huasheng.quant.open.gateway.sdk.vo.trade.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static com.huasheng.quant.open.gateway.sdk.constant.hq.HqRequestConstant.*;
import static com.huasheng.quant.open.gateway.sdk.constant.trade.TradeRequestConstant.*;

/**
 * 开箱即用API
 */
public class HSQuantOpenApiHandle {
    private static final Logger logger = LoggerFactory.getLogger(HSQuantOpenApiHandle.class);

    private final NotifyMessageRunnable notifyHandle;

    private final Gson gson;

    private volatile boolean isStopping = false;
    private volatile boolean stopped = false;

    private final Lock reconnectLock = new ReentrantLock();
    private final Lock stopLock = new ReentrantLock();

    private QuantOpenApi openApi = null;

    private ExecutorService executorService = null;

    public HSQuantOpenApiHandle(String ip, int httpPort, int tcpPort, NotifyMessageRunnable notifyHandle) {
        this.notifyHandle = notifyHandle;
        this.gson = new Gson();
        this.openApi = new QuantOpenApi(ip, httpPort, tcpPort, this.notifyHandle, this);
        this.executorService = new ThreadPoolExecutor(notifyHandle.getThreshold(), notifyHandle.getThreshold(),
                0L, TimeUnit.MILLISECONDS, new ArrayBlockingQueue<>(1));
    }

    public boolean startConnect() {
        // 推送消息处理守护线程
        for (int i = 0; i < notifyHandle.getThreshold(); i++) {
            executorService.submit(notifyHandle);
        }
        return connect();
    }

    private boolean connect() {
        try {
            // 【初始化连接和执行交易登录】
            openApi.init();

            if (!isActive()) {
                logger.error("Start/Restart client socket connect fail, please check.");
                throw new RuntimeException("初始化连接失败，请稍后重试！");
            }

            TimeUnit.SECONDS.sleep(1);
            return true;
        } catch (Exception e) {
            // 启动失败
            logger.error("Start/Restart client connect fail, please try again later.", e);
            return false;
        }
    }

    public boolean reconnect() {
        if (isActive()) {
            return true;
        }
        try {
            if (reconnectLock.tryLock(5, TimeUnit.SECONDS)) {
                try {
                    if (isActive()) {
                        return true;
                    }
                    if (isStopping || stopped) {
                        logger.info("The another thread isStopping/stopped client, unnecessary to reconnect.");
                        return false;
                    }
                    // 关闭资源
                    logger.info("Connection active status: {} before call reconnect.", isActive());
                    openApi.shutdown();
                    TimeUnit.SECONDS.sleep(2);
                    // 重连
                    logger.info("Starting reconnect server ...");
                    this.openApi.registerNewSocketApplication(new DefaultClientApplication(notifyHandle, this));
                    boolean connect = connect();
                    if (!connect) {
                        logger.warn("Reconnected server fail ip:port [{}:{}], please try again later.", this.openApi.getIp(), this.openApi.getTcpPort());
                    } else {
                        logger.info("Reconnected server success ip:port [{}:{}].", this.openApi.getIp(), this.openApi.getTcpPort());
                    }
                    return connect;
                } finally {
                    reconnectLock.unlock();
                }
            } else {
                logger.warn("The another thread is reconnecting server, please execute later.");
                return false;
            }
        } catch (Exception e) {
            logger.error("Reconnected gateway exception.", e);
            return false;
        }
    }

    public void disconnectAndStop() throws InterruptedException {
        if (stopped || isStopping) {
            logger.info("Open API handle isStopping/stopped, unnecessary to stop again.");
            return;
        }
        if (stopLock.tryLock(5, TimeUnit.SECONDS)) {
            try {
                isStopping = true;
                if (stopped) {
                    logger.info("Open API handle has stopped, unnecessary to stop again.");
                    return;
                }
                // 断开长连接
                if (openApi != null && isActive()) {
                    openApi.shutdown();
                }
                // 停止推送消息处理线程
                if (notifyHandle != null) {
                    this.notifyHandle.shutdown();
                }
                stopped = true;
            } finally {
                isStopping = false;
                stopLock.unlock();
            }
        } else {
            logger.warn("Open API handle is stopping, unnecessary to stop again.");
        }
    }

    public boolean isActive() {
        return openApi.isActive();
    }

    /* ********************************************   综合接口   ******************************************** */

    /**
     * 查询汇率列表
     *
     * @param rateType 汇率类型
     * @return
     */
    public Map<String, Map<String, String>> queryRateList(RateType rateType) {
        Map<String, Object> param = new HashMap<>();
        param.put("rateType", String.valueOf(rateType.getCode()));
        String jsonData = openApi.postMapRequest(OpenApiRequestConstant.HS_QUERY_RATE_LIST, param);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("HTTP queryRateList result empty.");
            return new HashMap<>();
        }
        HttpResult<Map<String, Map<String, String>>> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<Map<String, Map<String, String>>>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("HTTP queryRateList error: {}", jsonData);
            return new HashMap<>();
        }
        return httpResult.getData();
    }

    /* ********************************************   交易接口   ******************************************** */

    /**
     * 交易登录
     *
     * @param tradeLoginParam tradeLoginParam
     * @return boolean
     */
    public boolean tradeLogin(TradeLoginParam tradeLoginParam) {
        if (tradeLoginParam == null || StringUtils.isAnyBlank(tradeLoginParam.getPassword(), tradeLoginParam.getKeyBase64())) {
            logger.error("Trade login param is invalid.");
            return false;
        }

        String encryptedPwdBase64;
        try {
            byte[] decodedKey = Base64.getDecoder().decode(tradeLoginParam.getKeyBase64().getBytes(StandardCharsets.UTF_8));
            // use decodedKey as key, AES cipher encrypt password
            String originalPwd = tradeLoginParam.getPassword();

            // 创建密钥
            SecretKeySpec secretKeySpec = new SecretKeySpec(decodedKey, "AES");
            // 创建加密器
            Cipher cipher = Cipher.getInstance("AES/ECB/PKCS5Padding");
            // 初始化加密器
            cipher.init(Cipher.ENCRYPT_MODE, secretKeySpec);
            // 加密数据
            byte[] encryptedPwd = cipher.doFinal(originalPwd.getBytes());
            encryptedPwdBase64 = Base64.getEncoder().encodeToString(encryptedPwd);

        } catch (Exception e) {
            logger.error("Trade login encrypt password exception.", e);
            return false;
        }

        tradeLoginParam.setPassword(encryptedPwdBase64);
        tradeLoginParam.setKeyBase64(null);
        BaseQueryParam<TradeLoginParam> baseQueryParam = new BaseQueryParam<>(tradeLoginParam);

        String jsonData = openApi.postParamRequest(TradeRequestConstant.TRADE_LOGIN_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade login result empty.");
            return false;
        }

        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData, new TypeToken<HttpResult<CommonStringVo>>() {
        }.getType());
        if (httpResult != null && Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade login error: {}", jsonData);
            return false;
        }

        return true;
    }

    /**
     * 交易登出
     *
     * @return boolean
     */
    public boolean tradeLogout() {
        String jsonData = openApi.postMapRequest(TradeRequestConstant.TRADE_LOGOUT_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade logout result empty.");
            return false;
        }

        HttpResult<CommonBoolVo> httpResult = gson.fromJson(jsonData, new TypeToken<HttpResult<CommonBoolVo>>() {
        }.getType());
        if (httpResult != null && Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade logout error: {}", jsonData);
            return false;
        }

        return true;
    }

    /**
     * 查询客户资金信息
     */
    public ModelResult<MarginFundInfo> queryMarginFundInfo(ExchangeTypeParam exchangeTypeParam) {
        ModelResult<MarginFundInfo> modelResult = new ModelResult<>();
        if (exchangeTypeParam == null || StringUtils.isBlank(exchangeTypeParam.getExchangeType())) {
            logger.error("Trade query marginFundInfo param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<ExchangeTypeParam> baseQueryParam = new BaseQueryParam<>(exchangeTypeParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_MARGIN_FUND_INFO_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query marginFundInfo result empty.");
            return modelResult;
        }
        HttpResult<MarginFundInfo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<MarginFundInfo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query marginFundInfo error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询持仓列表
     */
    public ModelResult<HoldsVo> queryHoldsList(ExchangeTypeParam exchangeTypeParam) {
        ModelResult<HoldsVo> modelResult = new ModelResult<>();
        if (exchangeTypeParam == null || StringUtils.isBlank(exchangeTypeParam.getExchangeType())) {
            logger.error("Trade query holdsList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<ExchangeTypeParam> baseQueryParam = new BaseQueryParam<>(exchangeTypeParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_HOLDS_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query holdsList result empty.");
            return modelResult;
        }
        HttpResult<HoldsVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<HoldsVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query holdsList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询最大可买
     */
    public ModelResult<CommonIntVo> queryMaxBuyAmount(MaxBuySellAmountParam maxBuySellAmountParam) {
        ModelResult<CommonIntVo> modelResult = new ModelResult<>();
        if (maxBuySellAmountParam == null || StringUtils.isAnyBlank(maxBuySellAmountParam.getExchangeType(),
                maxBuySellAmountParam.getStockCode(), maxBuySellAmountParam.getEntrustPrice())) {
            logger.error("Trade query maxBuyAmount param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<MaxBuySellAmountParam> baseQueryParam = new BaseQueryParam<>(maxBuySellAmountParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_BUY_AMOUNT_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query maxBuyAmount result empty.");
            return modelResult;
        }
        HttpResult<CommonIntVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonIntVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query maxBuyAmount error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询最大可卖
     */
    public ModelResult<CommonIntVo> queryMaxSellAmount(MaxBuySellAmountParam maxBuySellAmountParam) {
        ModelResult<CommonIntVo> modelResult = new ModelResult<>();
        if (maxBuySellAmountParam == null || StringUtils.isAnyBlank(maxBuySellAmountParam.getExchangeType(),
                maxBuySellAmountParam.getStockCode())) {
            logger.error("Trade query maxSellAmount param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<MaxBuySellAmountParam> baseQueryParam = new BaseQueryParam<>(maxBuySellAmountParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_SELL_AMOUNT_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query maxSellAmount result empty.");
            return modelResult;
        }
        HttpResult<CommonIntVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonIntVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query maxSellAmount error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询当日资金流水
     */
    public ModelResult<FundJourVo> queryRealFundJourList(RealPageQueryParam realPageQueryParam) {
        ModelResult<FundJourVo> modelResult = new ModelResult<>();
        if (realPageQueryParam == null || StringUtils.isAnyBlank(realPageQueryParam.getExchangeType(), realPageQueryParam.getQueryParamStr())
                || realPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query realFundJourList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<RealPageQueryParam> baseQueryParam = new BaseQueryParam<>(realPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_REAL_FUND_JOUR_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query realFundJourList result empty.");
            return modelResult;
        }
        HttpResult<FundJourVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FundJourVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query realFundJourList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询历史资金流水
     */
    public ModelResult<FundJourVo> queryHistoryFundJourList(HistoryPageIntQueryParam historyPageQueryParam) {
        ModelResult<FundJourVo> modelResult = new ModelResult<>();
        if (historyPageQueryParam == null || StringUtils.isAnyBlank(historyPageQueryParam.getExchangeType(), historyPageQueryParam.getQueryParamStr())
                || historyPageQueryParam.getStartDate() == null || historyPageQueryParam.getEndDate() == null || historyPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query historyFundJourList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<HistoryPageIntQueryParam> baseQueryParam = new BaseQueryParam<>(historyPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_HISTORY_FUND_JOUR_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query historyFundJourList result empty.");
            return modelResult;
        }
        HttpResult<FundJourVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FundJourVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query historyFundJourList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询客户资金信息
     */
    public ModelResult<FuturesFundInfoVo> futuresQueryFundInfo() {
        ModelResult<FuturesFundInfoVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(FUTURES_QUERY_FUND_INFO_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query futuresFundInfo result empty.");
            return modelResult;
        }
        HttpResult<FuturesFundInfoVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesFundInfoVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query futuresFundInfo error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询持仓列表
     */
    public ModelResult<FuturesHoldsVo> futuresQueryHoldsList() {
        ModelResult<FuturesHoldsVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(FUTURES_QUERY_HOLDS_LIST_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query futuresHoldsList result empty.");
            return modelResult;
        }
        HttpResult<FuturesHoldsVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesHoldsVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query futuresHoldsList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询最大可买可卖
     */
    public ModelResult<FuturesMaxBuySellAmountVo> futuresQueryMaxBuySellAmount(FuturesCodeParam futuresCodeParam) {
        ModelResult<FuturesMaxBuySellAmountVo> modelResult = new ModelResult<>();
        if (futuresCodeParam == null || StringUtils.isBlank(futuresCodeParam.getStockCode())) {
            logger.error("Futures query maxBuySellAmount param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesCodeParam> baseQueryParam = new BaseQueryParam<>(futuresCodeParam);

        String jsonData = openApi.postParamRequest(FUTURES_QUERY_MAX_BUY_SELL_AMOUNT_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query maxBuySellAmount result empty.");
            return modelResult;
        }
        HttpResult<FuturesMaxBuySellAmountVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesMaxBuySellAmountVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query maxBuySellAmount error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询合约产品信息
     */
    public ModelResult<FuturesProductInfoVo> futuresQueryProductInfo(FuturesCodeListParam futuresCodeListParam) {
        ModelResult<FuturesProductInfoVo> modelResult = new ModelResult<>();
        if (futuresCodeListParam == null || CollectionUtils.isEmpty(futuresCodeListParam.getStockCode())) {
            logger.error("Futures query productInfo param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesCodeListParam> baseQueryParam = new BaseQueryParam<>(futuresCodeListParam);

        String jsonData = openApi.postParamRequest(FUTURES_QUERY_PRODUCT_INFO_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query productInfo result empty.");
            return modelResult;
        }
        HttpResult<FuturesProductInfoVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesProductInfoVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query productInfo error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 下单
     */
    public ModelResult<CommonStringVo> entrust(EntrustParam entrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (entrustParam == null || StringUtils.isAnyBlank(entrustParam.getStockCode(), entrustParam.getExchangeType(),
                entrustParam.getEntrustAmount(), entrustParam.getEntrustType(), entrustParam.getEntrustBs())) {
            logger.error("Trade entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (!OpenApiConstant.ENTRUST_COND_MARKET_TYPE_SET.contains(entrustParam.getEntrustType())
                && StringUtils.isBlank(entrustParam.getEntrustPrice())) {
            logger.error("Trade entrust fail, entrustPrice is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (OpenApiConstant.ENTRUST_COND_TYPE_SET.contains(entrustParam.getEntrustType())
                && StringUtils.isBlank(entrustParam.getSessionType())) {
            entrustParam.setSessionType(SessionType.COND_ORDER_DEFAULT.getCode());
        }
        BaseQueryParam<EntrustParam> baseQueryParam = new BaseQueryParam<>(entrustParam);

        String jsonData = openApi.postParamRequest(TRADE_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 撤单
     */
    public ModelResult<CommonStringVo> cancelEntrust(CancelEntrustParam cancelEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (cancelEntrustParam == null || StringUtils.isAnyBlank(cancelEntrustParam.getEntrustId(), cancelEntrustParam.getStockCode(),
                cancelEntrustParam.getExchangeType())) {
            logger.error("Trade cancel entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (StringUtils.isNotBlank(cancelEntrustParam.getEntrustType())
                && !StringUtils.equalsAny(cancelEntrustParam.getEntrustType(), OpenApiConstant.ENTRUST_COND_TYPE_ARRAY)) {
            logger.error("Trade cancel entrust fail, params entrustType is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<CancelEntrustParam> baseQueryParam = new BaseQueryParam<>(cancelEntrustParam);

        String jsonData = openApi.postParamRequest(TRADE_CANCEL_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade cancel entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade cancel entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 批量撤单
     */
    public ModelResult<BatchCancelEntrustVo> batchCancelEntrust(BatchCancelEntrustParam batchCancelEntrustParam) {
        ModelResult<BatchCancelEntrustVo> modelResult = new ModelResult<>();
        if (batchCancelEntrustParam == null || StringUtils.isBlank(batchCancelEntrustParam.getExchangeType())
                || CollectionUtils.isEmpty(batchCancelEntrustParam.getEntrustId())) {
            logger.error("Trade batch cancel entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<BatchCancelEntrustParam> baseQueryParam = new BaseQueryParam<>(batchCancelEntrustParam);

        String jsonData = openApi.postParamRequest(TRADE_BATCH_CANCEL_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade batch cancel entrust result empty.");
            return modelResult;
        }
        HttpResult<BatchCancelEntrustVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<BatchCancelEntrustVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade batch cancel entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 改单
     */
    public ModelResult<CommonStringVo> changeEntrust(ChangeEntrustParam changeEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (changeEntrustParam == null || StringUtils.isAnyBlank(changeEntrustParam.getEntrustId(), changeEntrustParam.getStockCode(),
                changeEntrustParam.getExchangeType(), changeEntrustParam.getEntrustAmount())) {
            logger.error("Trade change entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (!OpenApiConstant.ENTRUST_COND_MARKET_TYPE_SET.contains(changeEntrustParam.getEntrustType())
                && StringUtils.isBlank(changeEntrustParam.getEntrustPrice())) {
            logger.error("Trade change entrust fail, entrustPrice is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        // 校验条件单字段
        if (StringUtils.isNotBlank(changeEntrustParam.getEntrustType())
                && StringUtils.equalsAny(changeEntrustParam.getEntrustType(), OpenApiConstant.ENTRUST_COND_TYPE_ARRAY)
                && StringUtils.isBlank(changeEntrustParam.getCondValue())) {
            logger.error("Trade change entrust fail, params condValue is required.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<ChangeEntrustParam> baseQueryParam = new BaseQueryParam<>(changeEntrustParam);

        String jsonData = openApi.postParamRequest(TRADE_CHANGE_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade change entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade change entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询最大可用资产
     */
    public ModelResult<MaxAvailableAssetVo> queryMaxAvailableAsset(MaxAvailableAssetParam maxAvailableAssetParam) {
        ModelResult<MaxAvailableAssetVo> modelResult = new ModelResult<>();
        if (maxAvailableAssetParam == null || StringUtils.isAnyBlank(maxAvailableAssetParam.getExchangeType(), maxAvailableAssetParam.getStockCode(),
                maxAvailableAssetParam.getEntrustPrice()) || maxAvailableAssetParam.getEntrustType() == null) {
            logger.error("Trade query maxAvailableAsset param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<MaxAvailableAssetParam> baseQueryParam = new BaseQueryParam<>(maxAvailableAssetParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_MAX_AVAILABLE_ASSET_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query maxAvailableAsset result empty.");
            return modelResult;
        }
        HttpResult<MaxAvailableAssetVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<MaxAvailableAssetVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query maxAvailableAsset error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询当日委托
     */
    public ModelResult<OrderVo> queryRealEntrustList(RealEntrustPageQueryParam realEntrustPageQueryParam) {
        ModelResult<OrderVo> modelResult = new ModelResult<>();
        if (realEntrustPageQueryParam == null || StringUtils.isAnyBlank(realEntrustPageQueryParam.getExchangeType(), realEntrustPageQueryParam.getQueryParamStr())
                || realEntrustPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query realEntrustList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<RealEntrustPageQueryParam> baseQueryParam = new BaseQueryParam<>(realEntrustPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_REAL_ENTRUST_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query realEntrustList result empty.");
            return modelResult;
        }
        HttpResult<OrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query realEntrustList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询历史委托
     */
    public ModelResult<OrderVo> queryHistoryEntrustList(HistoryPageQueryParam historyPageQueryParam) {
        ModelResult<OrderVo> modelResult = new ModelResult<>();
        if (historyPageQueryParam == null || StringUtils.isAnyBlank(historyPageQueryParam.getExchangeType(), historyPageQueryParam.getQueryParamStr())
                || historyPageQueryParam.getStartDate() == null || historyPageQueryParam.getEndDate() == null || historyPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query historyEntrustList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<HistoryPageQueryParam> baseQueryParam = new BaseQueryParam<>(historyPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_HISTORY_ENTRUST_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query historyEntrustList result empty.");
            return modelResult;
        }
        HttpResult<OrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query historyEntrustList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询当日成交
     */
    public ModelResult<OrderVo> queryRealDeliverList(RealPageQueryParam realPageQueryParam) {
        ModelResult<OrderVo> modelResult = new ModelResult<>();
        if (realPageQueryParam == null || StringUtils.isAnyBlank(realPageQueryParam.getExchangeType(), realPageQueryParam.getQueryParamStr())
                || realPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query realDeliverList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<RealPageQueryParam> baseQueryParam = new BaseQueryParam<>(realPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_REAL_DELIVER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query realDeliverList result empty.");
            return modelResult;
        }
        HttpResult<OrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query realDeliverList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询历史成交
     */
    public ModelResult<OrderVo> queryHistoryDeliverList(HistoryPageQueryParam historyPageQueryParam) {
        ModelResult<OrderVo> modelResult = new ModelResult<>();
        if (historyPageQueryParam == null || StringUtils.isAnyBlank(historyPageQueryParam.getExchangeType(), historyPageQueryParam.getQueryParamStr(),
                historyPageQueryParam.getStartDate(), historyPageQueryParam.getEndDate()) || historyPageQueryParam.getQueryCount() == null) {
            logger.error("Trade query historyDeliverList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<HistoryPageQueryParam> baseQueryParam = new BaseQueryParam<>(historyPageQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_HISTORY_DELIVER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query historyDeliverList result empty.");
            return modelResult;
        }
        HttpResult<OrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query historyDeliverList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询当日条件单
     */
    public ModelResult<CondOrderVo> queryRealCondOrderList(RealPageNoQueryParam realPageNoQueryParam) {
        ModelResult<CondOrderVo> modelResult = new ModelResult<>();
        if (realPageNoQueryParam == null || StringUtils.isAnyBlank(realPageNoQueryParam.getExchangeType())
                || realPageNoQueryParam.getPageNo() == null || realPageNoQueryParam.getPageSize() == null) {
            logger.error("Trade query realCondOrderList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<RealPageNoQueryParam> baseQueryParam = new BaseQueryParam<>(realPageNoQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_REAL_COND_ORDER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query realCondOrderList result empty.");
            return modelResult;
        }
        HttpResult<CondOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CondOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query realCondOrderList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询历史条件单
     */
    public ModelResult<CondOrderVo> queryHistoryCondOrderList(HistoryPageNoQueryParam historyPageNoQueryParam) {
        ModelResult<CondOrderVo> modelResult = new ModelResult<>();
        if (historyPageNoQueryParam == null || StringUtils.isAnyBlank(historyPageNoQueryParam.getExchangeType(), historyPageNoQueryParam.getStartTime(),
                historyPageNoQueryParam.getEndTime()) || historyPageNoQueryParam.getPageNo() == null || historyPageNoQueryParam.getPageSize() == null) {
            logger.error("Trade query historyCondOrderList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<HistoryPageNoQueryParam> baseQueryParam = new BaseQueryParam<>(historyPageNoQueryParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_HISTORY_COND_ORDER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query historyCondOrderList result empty.");
            return modelResult;
        }
        HttpResult<CondOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CondOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query historyCondOrderList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询是否支持盘前盘后交易
     */
    public ModelResult<CommonStringVo> queryBeforeAndAfterSupport(StockCodeParam stockCodeParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (stockCodeParam == null || StringUtils.isAnyBlank(stockCodeParam.getExchangeType(), stockCodeParam.getStockCode())) {
            logger.error("Trade query beforeAndAfterSupport param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<StockCodeParam> baseQueryParam = new BaseQueryParam<>(stockCodeParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_BEFORE_AND_AFTER_SUPPORT_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query beforeAndAfterSupport result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query beforeAndAfterSupport error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询股票沽空信息
     */
    public ModelResult<StockShortInfoVo> queryStockShortInfo(StockCodeParam stockCodeParam) {
        ModelResult<StockShortInfoVo> modelResult = new ModelResult<>();
        if (stockCodeParam == null || StringUtils.isAnyBlank(stockCodeParam.getExchangeType(), stockCodeParam.getStockCode())) {
            logger.error("Trade query stockShortInfo param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<StockCodeParam> baseQueryParam = new BaseQueryParam<>(stockCodeParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_STOCK_SHORT_INFO_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query stockShortInfo result empty.");
            return modelResult;
        }
        HttpResult<StockShortInfoVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<StockShortInfoVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query stockShortInfo error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询股票融资融券数据
     */
    public ModelResult<MarginFullInfoVo> queryMarginFullInfo(DataTypeParam dataTypeParam) {
        ModelResult<MarginFullInfoVo> modelResult = new ModelResult<>();
        if (dataTypeParam == null || StringUtils.isAnyBlank(dataTypeParam.getStockCode(), dataTypeParam.getDataType())) {
            logger.error("Trade query marginFullInfo param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<DataTypeParam> baseQueryParam = new BaseQueryParam<>(dataTypeParam);

        String jsonData = openApi.postParamRequest(TRADE_QUERY_MARGIN_FULL_INFO_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade query marginFullInfo result empty.");
            return modelResult;
        }
        HttpResult<MarginFullInfoVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<MarginFullInfoVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade query marginFullInfo error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-下单
     */
    public ModelResult<CommonStringVo> futuresEntrust(FuturesEntrustParam futuresEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (futuresEntrustParam == null || StringUtils.isAnyBlank(futuresEntrustParam.getStockCode(), futuresEntrustParam.getEntrustType(),
                futuresEntrustParam.getEntrustPrice(), futuresEntrustParam.getEntrustAmount(), futuresEntrustParam.getEntrustBs(),
                futuresEntrustParam.getValidTimeType())) {
            logger.error("Futures entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (futuresEntrustParam.getValidTimeType().equals(FuturesValidTimeType.VALID_ON_SPECIFIED_DATE.getCode())
                && StringUtils.isBlank(futuresEntrustParam.getValidTime())) {
            logger.error("Futures entrust fail, validTime must not be null.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesEntrustParam> baseQueryParam = new BaseQueryParam<>(futuresEntrustParam);

        String jsonData = openApi.postParamRequest(FUTURES_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-撤单
     */
    public ModelResult<CommonStringVo> futuresCancelEntrust(FuturesEntrustIdParam futuresEntrustIdParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (futuresEntrustIdParam == null || StringUtils.isAnyBlank(futuresEntrustIdParam.getStockCode(), futuresEntrustIdParam.getEntrustId())) {
            logger.error("Futures cancel entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesEntrustIdParam> baseQueryParam = new BaseQueryParam<>(futuresEntrustIdParam);

        String jsonData = openApi.postParamRequest(FUTURES_CANCEL_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures cancel entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures cancel entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-改单
     */
    public ModelResult<CommonStringVo> futuresModifyEntrust(FuturesChangeEntrustParam futuresChangeEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (futuresChangeEntrustParam == null || StringUtils.isAnyBlank(futuresChangeEntrustParam.getStockCode(), futuresChangeEntrustParam.getEntrustId())) {
            logger.error("Futures modify entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (futuresChangeEntrustParam.getValidTimeType().equals(FuturesValidTimeType.VALID_ON_SPECIFIED_DATE.getCode())
                && StringUtils.isBlank(futuresChangeEntrustParam.getValidTime())) {
            logger.error("Futures entrust fail, params is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesChangeEntrustParam> baseQueryParam = new BaseQueryParam<>(futuresChangeEntrustParam);

        String jsonData = openApi.postParamRequest(FUTURES_MODIFY_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures modify entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures modify entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询当日委托
     */
    public ModelResult<FuturesOrderVo> futuresQueryRealEntrustList() {
        ModelResult<FuturesOrderVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(FUTURES_QUERY_REAL_ENTRUST_LIST_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query realEntrustList result empty.");
            return modelResult;
        }
        HttpResult<FuturesOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query realEntrustList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询历史委托
     */
    public ModelResult<FuturesHistoryOrderVo> futuresQueryHistoryEntrustList(FuturesPageQueryParam futuresPageQueryParam) {
        ModelResult<FuturesHistoryOrderVo> modelResult = new ModelResult<>();
        if (futuresPageQueryParam == null || StringUtils.isAnyBlank(futuresPageQueryParam.getStartDate(), futuresPageQueryParam.getEndDate())
                || futuresPageQueryParam.getPageNo() == null || futuresPageQueryParam.getPageSize() == null) {
            logger.error("Futures query historyEntrustList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesPageQueryParam> baseQueryParam = new BaseQueryParam<>(futuresPageQueryParam);

        String jsonData = openApi.postParamRequest(FUTURES_QUERY_HISTORY_ENTRUST_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query historyEntrustList result empty.");
            return modelResult;
        }
        HttpResult<FuturesHistoryOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesHistoryOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query historyEntrustList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询当日成交
     */
    public ModelResult<FuturesOrderVo> futuresQueryRealDeliverList() {
        ModelResult<FuturesOrderVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(FUTURES_QUERY_REAL_DELIVER_LIST_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query realDeliverList result empty.");
            return modelResult;
        }
        HttpResult<FuturesOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query realDeliverList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 期货-查询历史成交
     */
    public ModelResult<FuturesHistoryOrderVo> futuresQueryHistoryDeliverList(FuturesPageQueryParam futuresPageQueryParam) {
        ModelResult<FuturesHistoryOrderVo> modelResult = new ModelResult<>();
        if (futuresPageQueryParam == null || StringUtils.isAnyBlank(futuresPageQueryParam.getStartDate(), futuresPageQueryParam.getEndDate())
                || futuresPageQueryParam.getPageNo() == null || futuresPageQueryParam.getPageSize() == null) {
            logger.error("Futures query historyDeliverList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<FuturesPageQueryParam> baseQueryParam = new BaseQueryParam<>(futuresPageQueryParam);

        String jsonData = openApi.postParamRequest(FUTURES_QUERY_HISTORY_DELIVER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Futures query historyDeliverList result empty.");
            return modelResult;
        }
        HttpResult<FuturesHistoryOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<FuturesHistoryOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Futures query historyDeliverList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-下单
     */
    public ModelResult<CommonStringVo> algoEntrust(AlgoEntrustParam algoEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (algoEntrustParam == null || StringUtils.isAnyBlank(algoEntrustParam.getStockCode(), algoEntrustParam.getExchangeType(),
                algoEntrustParam.getEntrustType(), algoEntrustParam.getEntrustPrice(), algoEntrustParam.getEntrustAmount(), algoEntrustParam.getEntrustBs(),
                algoEntrustParam.getTargetStrategy(), algoEntrustParam.getSessionType())) {
            logger.error("Trade algo entrust param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        if (StringUtils.isAnyBlank(algoEntrustParam.getStrategyParam().getMaxVolume(), algoEntrustParam.getStrategyParam().getSensitivity())) {
            logger.error("Algo add order fail, strategy params is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoEntrustParam> baseQueryParam = new BaseQueryParam<>(algoEntrustParam);

        String jsonData = openApi.postParamRequest(ALGO_ADD_ORDER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo entrust result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo entrust error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-撤销母单
     */
    public ModelResult<CommonStringVo> algoCancelOrder(AlgoOrderIdParam algoOrderIdParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (algoOrderIdParam == null || StringUtils.isAnyBlank(algoOrderIdParam.getExchangeType(), algoOrderIdParam.getOrderId())) {
            logger.error("Trade algo cancel order param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoOrderIdParam> baseQueryParam = new BaseQueryParam<>(algoOrderIdParam);

        String jsonData = openApi.postParamRequest(ALGO_CANCEL_ORDER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo cancel order result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo cancel order error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-撤销子单
     */
    public ModelResult<CommonStringVo> algoCancelSubOrder(AlgoCancelEntrustParam algoCancelEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (algoCancelEntrustParam == null || StringUtils.isAnyBlank(algoCancelEntrustParam.getExchangeType(), algoCancelEntrustParam.getOrderId(),
                algoCancelEntrustParam.getEntrustId())) {
            logger.error("Trade algo cancel sub order param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoCancelEntrustParam> baseQueryParam = new BaseQueryParam<>(algoCancelEntrustParam);

        String jsonData = openApi.postParamRequest(ALGO_CANCEL_ENTRUST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo cancel sub order result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo cancel sub order error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-改单
     */
    public ModelResult<CommonStringVo> algoModifyOrder(AlgoChangeEntrustParam algoChangeEntrustParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (algoChangeEntrustParam == null || StringUtils.isAnyBlank(algoChangeEntrustParam.getOrderId(), algoChangeEntrustParam.getEntrustPrice(),
                algoChangeEntrustParam.getEntrustAmount(), algoChangeEntrustParam.getStockCode(), algoChangeEntrustParam.getExchangeType())) {
            logger.error("Trade algo change order param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoChangeEntrustParam> baseQueryParam = new BaseQueryParam<>(algoChangeEntrustParam);

        String jsonData = openApi.postParamRequest(ALGO_CHANGE_ORDER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo change order result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo change order error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-操作
     */
    public ModelResult<CommonStringVo> algoOrderAction(AlgoActionParam algoActionParam) {
        ModelResult<CommonStringVo> modelResult = new ModelResult<>();
        if (algoActionParam == null || StringUtils.isAnyBlank(algoActionParam.getExchangeType(), algoActionParam.getOrderId(),
                algoActionParam.getAction())) {
            logger.error("Trade algo action order param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoActionParam> baseQueryParam = new BaseQueryParam<>(algoActionParam);

        String jsonData = openApi.postParamRequest(ALGO_ACTION_ORDER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo action order result empty.");
            return modelResult;
        }
        HttpResult<CommonStringVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonStringVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo action order error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-查询母单列表
     */
    public ModelResult<AlgoOrderVo> algoQueryOrderList(AlgoQueryPageParam algoQueryPageParam) {
        ModelResult<AlgoOrderVo> modelResult = new ModelResult<>();
        if (algoQueryPageParam == null || StringUtils.isAnyBlank(algoQueryPageParam.getStartDate(), algoQueryPageParam.getEndDate())
                || algoQueryPageParam.getPageNo() == null || algoQueryPageParam.getPageSize() == null) {
            logger.error("Trade algo query orderList param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoQueryPageParam> baseQueryParam = new BaseQueryParam<>(algoQueryPageParam);

        String jsonData = openApi.postParamRequest(ALGO_QUERY_ORDER_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo query orderList result empty.");
            return modelResult;
        }
        HttpResult<AlgoOrderVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<AlgoOrderVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo query orderList error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 策略单-查询子单
     */
    public ModelResult<AlgoEntrustIdVo> algoQuerySubOrderIdList(AlgoQueryEntrustIdParam algoQueryEntrustIdParam) {
        ModelResult<AlgoEntrustIdVo> modelResult = new ModelResult<>();
        if (algoQueryEntrustIdParam == null || StringUtils.isAnyBlank(algoQueryEntrustIdParam.getExchangeType(),
                algoQueryEntrustIdParam.getOrderId(), algoQueryEntrustIdParam.getTradeDate())) {
            logger.error("Trade algo query subOrderId param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<AlgoQueryEntrustIdParam> baseQueryParam = new BaseQueryParam<>(algoQueryEntrustIdParam);

        String jsonData = openApi.postParamRequest(ALGO_QUERY_ENTRUST_ID_LIST_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade algo query subOrderId result empty.");
            return modelResult;
        }
        HttpResult<AlgoEntrustIdVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<AlgoEntrustIdVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade algo query subOrderId error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 订阅交易推送
     */
    public ModelResult<CommonBoolVo> subscribe() {
        ModelResult<CommonBoolVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(TRADE_SUBSCRIBE_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade subscribe result empty.");
            return modelResult;
        }
        HttpResult<CommonBoolVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonBoolVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade subscribe error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 取消订阅交易推送
     */
    public ModelResult<CommonBoolVo> unsubscribe() {
        ModelResult<CommonBoolVo> modelResult = new ModelResult<>();
        String jsonData = openApi.postMapRequest(TRADE_UNSUBSCRIBE_REQUEST_MSG_TYPE, Maps.newHashMap());
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Trade unsubscribe result empty.");
            return modelResult;
        }
        HttpResult<CommonBoolVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<CommonBoolVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Trade unsubscribe error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /* ********************************************   行情接口   ******************************************** */

    /**
     * 查询基础报价
     */
    public ModelResult<BasicQotVo> queryHqBasicQot(HqBasicQotParam hqBasicQotParam) {
        ModelResult<BasicQotVo> modelResult = new ModelResult<>();
        if (hqBasicQotParam == null || CollectionUtils.isEmpty(hqBasicQotParam.getSecurity())) {
            logger.error("Hq query basicQot param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }
        BaseQueryParam<HqBasicQotParam> baseQueryParam = new BaseQueryParam<>(hqBasicQotParam);

        String jsonData = openApi.postParamRequest(HQ_BASIC_QOT_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query basicQot result empty.");
            return modelResult;
        }
        HttpResult<BasicQotVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<BasicQotVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query basicQot error: {}", jsonData);
            return modelResult;
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询买卖经纪摆盘
     */
    public ModelResult<BrokerVo> queryHqBroker(SecurityParam securityParam) {
        ModelResult<BrokerVo> modelResult = new ModelResult<>();
        if (securityParam == null) {
            logger.error("Hq query broker param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<SecurityParam> baseQueryParam = new BaseQueryParam<>(securityParam);
        String jsonData = openApi.postParamRequest(HQ_BROKER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query broker result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<BrokerVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<BrokerVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query broker error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询买卖档
     */
    public ModelResult<OrderBookVo> queryHqOrderBook(OrderBookParam orderBookParam) {
        ModelResult<OrderBookVo> modelResult = new ModelResult<>();
        if (orderBookParam == null) {
            logger.error("Hq query orderBook param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<OrderBookParam> baseQueryParam = new BaseQueryParam<>(orderBookParam);
        String jsonData = openApi.postParamRequest(HQ_ORDER_BOOK_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query orderBook result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<OrderBookVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OrderBookVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query orderBook error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询逐笔
     */
    public ModelResult<TickerVo> queryHqTicker(TickerParam tickerParam) {
        ModelResult<TickerVo> modelResult = new ModelResult<>();
        if (tickerParam == null) {
            logger.error("Hq query ticker param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<TickerParam> baseQueryParam = new BaseQueryParam<>(tickerParam);
        String jsonData = openApi.postParamRequest(HQ_TICKER_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query ticker result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<TickerVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<TickerVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query ticker error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询K线
     */
    public ModelResult<KLineVo> queryHqKLine(KLineParam kLineParam) {
        ModelResult<KLineVo> modelResult = new ModelResult<>();
        if (kLineParam == null) {
            logger.error("Hq query kline param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<KLineParam> baseQueryParam = new BaseQueryParam<>(kLineParam);
        String jsonData = openApi.postParamRequest(HQ_KL_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query kline result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<KLineVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<KLineVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query kline error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询分时
     */
    public ModelResult<TimeShareVo> queryHqTimeShare(TimeShareParam securityParam) {
        ModelResult<TimeShareVo> modelResult = new ModelResult<>();
        if (securityParam == null) {
            logger.error("Hq query timeShare param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<SecurityParam> baseQueryParam = new BaseQueryParam<>(securityParam);
        String jsonData = openApi.postParamRequest(HQ_TIME_SHARE_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query timeShare result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<TimeShareVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<TimeShareVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query timeShare error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询期权链到期日列表
     */
    public ModelResult<OptionChainExpireDateVo> queryHqUsOptionChainExpireDate(OptionParam optionParam) {
        ModelResult<OptionChainExpireDateVo> modelResult = new ModelResult<>();
        if (optionParam == null) {
            logger.error("Hq query optionChainExpireDate param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<OptionParam> baseQueryParam = new BaseQueryParam<>(optionParam);
        String jsonData = openApi.postParamRequest(HQ_US_OPTION_CHAIN_EXPIRE_DATE_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query optionChainExpireDate result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<OptionChainExpireDateVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OptionChainExpireDateVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query optionChainExpireDate error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 查询期权链Code列表
     */
    public ModelResult<OptionChainCodeVo> queryHqUsOptionChainCode(OptionChainParam optionParam) {
        ModelResult<OptionChainCodeVo> modelResult = new ModelResult<>();
        if (optionParam == null) {
            logger.error("Hq query optionChainCode param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<OptionChainParam> baseQueryParam = new BaseQueryParam<>(optionParam);
        String jsonData = openApi.postParamRequest(HQ_US_OPTION_CHAIN_CODE_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq query optionChainCode result empty.");
            return modelResult.withModel(null);
        }
        HttpResult<OptionChainCodeVo> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<OptionChainCodeVo>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq query optionChainCode error: {}", jsonData);
            return modelResult.withModel(null);
        }
        return modelResult.withModel(httpResult.getData());
    }

    /**
     * 订阅行情推送
     */
    public ModelResult<Boolean> hqSubscribe(HqSubscribeParam hqSubscribeParam) {
        ModelResult<Boolean> modelResult = new ModelResult<>();
        if (hqSubscribeParam == null) {
            logger.error("Hq subscribe param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<HqSubscribeParam> baseQueryParam = new BaseQueryParam<>(hqSubscribeParam);
        String jsonData = openApi.postParamRequest(HQ_SUBSCRIBE_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq subscribe result empty.");
            return modelResult.withModel(Boolean.FALSE);
        }
        HttpResult<Boolean> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<Boolean>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq subscribe error: {}", jsonData);
            return modelResult.withModel(Boolean.FALSE);
        }
        return modelResult.withModel(httpResult.getOk());
    }

    /**
     * 取消订阅行情推送
     */
    public ModelResult<Boolean> hqUnsubscribe(HqUnsubscribeParam hqUnsubscribeParam) {
        ModelResult<Boolean> modelResult = new ModelResult<>();
        if (hqUnsubscribeParam == null) {
            logger.error("Hq unsubscribe param is invalid.");
            return modelResult.withError(ISystemError.PARAM_INVALID.getCode(), ISystemError.PARAM_INVALID.getDescription());
        }

        BaseQueryParam<HqUnsubscribeParam> baseQueryParam = new BaseQueryParam<>(hqUnsubscribeParam);
        String jsonData = openApi.postParamRequest(HQ_UNSUBSCRIBE_REQUEST_MSG_TYPE, baseQueryParam);
        if (StringUtils.isBlank(jsonData)) {
            logger.error("Hq unsubscribe result empty.");
            return modelResult.withModel(Boolean.FALSE);
        }
        HttpResult<Boolean> httpResult = gson.fromJson(jsonData,
                new TypeToken<HttpResult<Boolean>>() {
                }.getType());
        if (Boolean.FALSE.equals(httpResult.getOk())) {
            logger.error("Hq unsubscribe error: {}", jsonData);
            return modelResult.withModel(Boolean.FALSE);
        }
        return modelResult.withModel(httpResult.getOk());
    }
}
