package com.huasheng.quant.open.gateway.api;

import com.huasheng.quant.open.gateway.sdk.mapper.HqMapper;
import com.huasheng.quant.open.gateway.sdk.mapper.TradeMapper;
import com.huasheng.quant.open.gateway.sdk.notify.IMessageHandler;
import com.huasheng.quant.open.gateway.sdk.notify.NotifyMessageRunnable;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.constant.NotifyMsgTypeProto;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.NotifyProto;
import com.huasheng.quant.open.gateway.sdk.protobuf.hq.notify.BasicQotNotifyProto.BasicQotNotify;
import com.huasheng.quant.open.gateway.sdk.protobuf.hq.notify.BrokerNotifyProto.BrokerNotify;
import com.huasheng.quant.open.gateway.sdk.protobuf.hq.notify.OrderBookFullNotifyProto;
import com.huasheng.quant.open.gateway.sdk.protobuf.hq.notify.TickerNotifyProto;
import com.huasheng.quant.open.gateway.sdk.protobuf.trade.notify.TradeStockDeliverNotifyProto.TradeStockDeliverNotify;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufNofityMessage;
import com.huasheng.quant.open.gateway.sdk.vo.hq.BasicQot;
import com.huasheng.quant.open.gateway.sdk.vo.hq.Broker;
import com.huasheng.quant.open.gateway.sdk.vo.hq.OrderBook;
import com.huasheng.quant.open.gateway.sdk.vo.hq.Security;
import com.huasheng.quant.open.gateway.sdk.vo.hq.Ticker;
import com.huasheng.quant.open.gateway.sdk.vo.notify.HqBasicQotNotifyMessageVo;
import com.huasheng.quant.open.gateway.sdk.vo.notify.HqBrokerNotifyMessageVo;
import com.huasheng.quant.open.gateway.sdk.vo.notify.HqOrderBookNotifyMessageVo;
import com.huasheng.quant.open.gateway.sdk.vo.notify.HqTickerNotifyMessageVo;
import com.huasheng.quant.open.gateway.sdk.vo.notify.TradeNotifyMessageVo;
import org.apache.commons.collections4.CollectionUtils;

import java.util.List;

/**
 * 处理消息推送继承类
 */
public class PushNotifyMessageHandler extends NotifyMessageRunnable implements IMessageHandler {

    private final TradeMapper tradeMapper = TradeMapper.INSTANCE;

    private final HqMapper hqMapper = HqMapper.INSTANCE;

    /**
     * @param threshold 消费线程池大小
     * @param queueSize 内存队列大小
     * @param waitTime  写入等待时间（秒）
     */
    public PushNotifyMessageHandler(int threshold, int queueSize, long waitTime) {
        super(threshold, queueSize, waitTime);
    }

    /**
     * 消息处理方法
     *
     * @param notifyMessage
     */
    @Override
    public void notifyMessageHandle(ProtoBufNofityMessage notifyMessage) {
        NotifyProto.PBNotify pbNotify = notifyMessage.getMsgBody();
        // logger.info("****** Consume notify message ****** notifyType: {}, notifyId: {}", pbNotify.getNotifyMsgType(), pbNotify.getNotifyId());
        try {
            // 交易和期货回报推送
            if (pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.TradeStockDeliverMsgType
                    || pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.FuturesTradeStockDeliverMsgType) {
                TradeNotifyMessageVo tradeNotifyMessageVo = tradeMapper.toTradeNotifyMessageVo(
                        TradeStockDeliverNotify.parseFrom(pbNotify.getPayload().getValue()));
                tradeNotifyMessageHandle(tradeNotifyMessageVo);
                return;
            }
            // 行情基础报价消息推送
            if (pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.BasicQotNotifyMsgType) {
                HqBasicQotNotifyMessageVo hqBasicQotNotifyMessageVo = hqMapper.toHqBasicQotNotifyMessageVo(
                        BasicQotNotify.parseFrom(pbNotify.getPayload().getValue()));
                hqBasicQotNotifyMessageHandle(hqBasicQotNotifyMessageVo);
                return;
            }
            // 行情买卖经纪消息推送
            if (pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.BrokerQueueNotifyMsgType) {
                HqBrokerNotifyMessageVo hqBasicQotNotifyMessageVo = hqMapper.toHqBrokerNotifyMessageVo(
                        BrokerNotify.parseFrom(pbNotify.getPayload().getValue()));
                hqBrokerNotifyMessageHandle(hqBasicQotNotifyMessageVo);
                return;
            }
            // 行情买卖档消息推送
            if (pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.OrderBookNotifyMsgType) {
                HqOrderBookNotifyMessageVo hqOrderBookNotifyMessageVo = hqMapper.toHqOrderBookNotifyMessageVo(
                        OrderBookFullNotifyProto.OrderBookFullNotify.parseFrom(pbNotify.getPayload().getValue()));
                hqOrderBookNotifyMessageHandle(hqOrderBookNotifyMessageVo);
                return;
            }
            // 行情逐笔消息推送
            if (pbNotify.getNotifyMsgType() == NotifyMsgTypeProto.NotifyMsgType.TickerNotifyMsgType) {
                HqTickerNotifyMessageVo hqTickerNotifyMessageVo = hqMapper.toHqTickerNotifyMessageVo(
                        TickerNotifyProto.TickerNotify.parseFrom(pbNotify.getPayload().getValue()));
                hqTickerNotifyMessageHandle(hqTickerNotifyMessageVo);
            }
        } catch (Exception e) {
            logger.error("Handle notify message exception, notifyType: {}, notifyId: {}", pbNotify.getNotifyMsgType(),
                    pbNotify.getNotifyId(), e);
        }
    }

    /**
     * 交易回报消息处理
     *
     * @param notify
     */
    @Override
    public void tradeNotifyMessageHandle(TradeNotifyMessageVo notify) {
        logger.info("Trade notify message, orderId: {}, stockCode: {}, entrustStatus : {}, remark: {}", notify.getRecordNo(),
                notify.getStockCode(), notify.getEntrustStatus(), notify.getRemark());
    }

    /**
     * 行情基础报价消息处理
     *
     * @param notify
     */
    @Override
    public void hqBasicQotNotifyMessageHandle(HqBasicQotNotifyMessageVo notify) {
        Security security = notify.getSecurity();
        BasicQot basicQot = notify.getBasicQot();
        if (security != null && basicQot != null) {
            logger.info("Hq basicQot notify message, {} stockCode: {}, lastPrice : {}, volumeStr: {}", Thread.currentThread().getName(), 
                    security.getCode(), basicQot.getLastPrice(), basicQot.getVolumeStr());
        }
    }

    /**
     * 行情买卖经纪消息处理
     *
     * @param notify
     */
    @Override
    public void hqBrokerNotifyMessageHandle(HqBrokerNotifyMessageVo notify) {
        Security security = notify.getSecurity();
        List<Broker> brokerList = notify.getBrokerList();
        if (security != null && CollectionUtils.isNotEmpty(brokerList)) {
            for (Broker broker : brokerList) {
                if (broker.getLevel() == 1) {
                    logger.info("Hq broker notify message, stockCode: {}, level : {}, name: {}, type: {}", security.getCode(),
                            broker.getLevel(), broker.getName(), broker.getType());
                }
            }
        }
    }

    /**
     * 行情买卖档消息处理
     *
     * @param notify
     */
    @Override
    public void hqOrderBookNotifyMessageHandle(HqOrderBookNotifyMessageVo notify) {
        Security security = notify.getSecurity();
        List<OrderBook> orderBookBidList = notify.getOrderBookBidList();
        if (security != null && CollectionUtils.isNotEmpty(orderBookBidList)) {
            for (OrderBook orderBook : orderBookBidList) {
                if (orderBook.getLevel() == 1) {
                    logger.info("Hq orderBook notify message, stockCode: {}, level : {}, price: {}, volume: {}, orderCount: {}",
                            security.getCode(), orderBook.getLevel(), orderBook.getPrice(), orderBook.getVolume(), orderBook.getOrderCount());
                }
            }
        }
    }

    /**
     * 行情逐笔消息处理
     *
     * @param notify
     */
    @Override
    public void hqTickerNotifyMessageHandle(HqTickerNotifyMessageVo notify) {
        Security security = notify.getSecurity();
        Ticker ticker = notify.getTicker();
        if (security != null && ticker != null) {
            String mktTmType = "";
            switch (ticker.getMktTmType())
            {
                case 1:
                    mktTmType = "(MktTmType.IN_MARKET)";
                    break;
                case -1:
                    mktTmType = "(MktTmType.PRE_MARKET)";
                    break;
                case -2:
                    mktTmType = "(MktTmType.AFTER_MARKET)";
                    break;
            }
            logger.info("Hq ticker notify message{}, stockCode: {}, type : {}, side: {}, price: {}, timestamp: {}",mktTmType,
                    security.getCode(), ticker.getType(), ticker.getSide(), ticker.getPrice(), ticker.getTimestamp());
        }
    }

}
