package com.huasheng.quant.open.gateway.api;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.huasheng.quant.open.gateway.sdk.handlers.Application;
import com.huasheng.quant.open.gateway.sdk.handlers.DefaultClientApplication;
import com.huasheng.quant.open.gateway.sdk.netty.pipeline.client.SocketClient;
import com.huasheng.quant.open.gateway.sdk.notify.NotifyMessageRunnable;
import com.huasheng.quant.open.gateway.sdk.protocol.session.Session;
import com.huasheng.quant.open.gateway.sdk.utils.HttpClientUtil;
import com.huasheng.quant.open.gateway.sdk.vo.RequestParam;
import io.netty.channel.Channel;
import io.netty.util.Attribute;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;
import java.util.Optional;

import static com.huasheng.quant.open.gateway.sdk.constant.OpenApiRequestConstant.COLON;
import static com.huasheng.quant.open.gateway.sdk.constant.OpenApiRequestConstant.HTTP_PROTOCOL_PREFIX;

public class QuantOpenApi {

    protected static final Logger logger = LoggerFactory.getLogger(QuantOpenApi.class);

    protected long timeoutSeconds = 10;

    private final Gson gson;

    private String ip;

    private int httpPort;

    /**
     * http://127.0.0.1:11111
     */
    private String domain;

    private int tcpPort;

    protected final SocketClient client;

    public QuantOpenApi(String ip, int httpPort, int tcpPort, NotifyMessageRunnable notifyHandle) {
        this(ip, httpPort, tcpPort, new DefaultClientApplication(notifyHandle, null));
    }

    public QuantOpenApi(String ip, int httpPort, int tcpPort, NotifyMessageRunnable notifyHandle, HSQuantOpenApiHandle hsQuantOpenApiHandle) {
        this(ip, httpPort, tcpPort, new DefaultClientApplication(notifyHandle, hsQuantOpenApiHandle));
    }

    public QuantOpenApi(String ip, int httpPort, int tcpPort, Application application) {
        this.ip = ip;
        this.httpPort = httpPort;
        this.domain = HTTP_PROTOCOL_PREFIX + ip + COLON + httpPort;
        this.tcpPort = tcpPort;
        this.gson = new Gson();
        this.client = new SocketClient(application);
    }

    public String getIp() {
        return ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public int getTcpPort() {
        return tcpPort;
    }

    public void setTcpPort(int tcpPort) {
        this.tcpPort = tcpPort;
    }

    /**
     * 初始化长连接
     *
     * @return
     * @throws Exception
     */
    public Channel init() throws Exception {
        if (StringUtils.isBlank(ip) || tcpPort == 0) {
            logger.error("Start/Restart client connect fail, please check ip and tcpPort.");
            return null;
        }
        Channel channel = client.connect(ip, tcpPort);
        logger.info("Init client socket connection active status: {}", channel.isActive());
        // Socket登录初始化
        Session session = Session.newBuilder()
                .createTime(System.currentTimeMillis())
                .clientIp("127.0.0.1")
                .channelId(channel.id().asShortText())
                .deviceNo("127.0.0.1")
                .build();
        session.setNextIncomingMessageSeqNum(0);
        session.setNextOutgoingMessageSeqNum(0);

        Attribute<Session> fixSessionAttribute = channel.attr(Session.SESSION_KEY);
        fixSessionAttribute.compareAndSet(null, session);

        logger.info("FixClient is started and connected to {}", channel.remoteAddress());
        return channel;
    }

    public void shutdown() {
        try {
            client.disconnect();
        } catch (Exception e) {
            logger.warn("Socket close resource before reconnect server exception: {}", e.getMessage());
        }
    }

    public boolean isActive() {
        return client.isActive();
    }

    public void registerNewSocketApplication(Application application) {
        client.setSocketApplication(application);
    }

    public String postMapRequest(String path, Map<String, Object> param) {
        Map<String, Object> map = Maps.newHashMap();
        map.put("timeout_sec", timeoutSeconds);
        map.put("params", param);
        String paramStr = gson.toJson(map);
        return postRequest(domain + path, paramStr);
    }

    public String postParamRequest(String path, RequestParam param) {
        String paramStr = gson.toJson(param);
        return postRequest(domain + path, paramStr);
    }

    private String postRequest(String url, String requestParam) {
        try {
            Optional<String> postBody = HttpClientUtil.postBody(url, requestParam);
            if (!postBody.isPresent()) {
                return null;
            }
            String result = postBody.get();
            if (StringUtils.isBlank(result)) {
                return null;
            }
            return result;
        } catch (Exception e) {
            logger.error("", e);
            return StringUtils.EMPTY;
        }
    }

}
