package com.huasheng.quant.open.gateway.sdk.netty.codec;

import com.google.protobuf.MessageLite;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.HeartBeatProto.HeartBeat;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.NotifyProto.PBNotify;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.RequestProto.PBRequest;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.ResponseProto.PBResponse;
import com.huasheng.quant.open.gateway.sdk.protocol.MessageHeader;
import com.huasheng.quant.open.gateway.sdk.protocol.MsgType;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufHeartBeatMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufNofityMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufRequestMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufResponseMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoFmtType;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.utils.RsaSignature;
import io.netty.buffer.ByteBuf;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;

public class MessageCodecHelper {

    protected static final Logger logger = LoggerFactory.getLogger(MessageCodecHelper.class);

    private static final byte[] emptyBodySHA1 = new byte[128];

    /**
     * 解析报文头
     *
     * @param in
     * @return
     */
    public static MessageHeader parseHeader(ByteBuf in) {
        CharSequence szHeaderFlag = in.readCharSequence(2, StandardCharsets.UTF_8);// 2
        short msgType = in.readShortLE();// 2
        byte protoFmtType = in.readByte();// 1
        byte protoVer = in.readByte();// 1
        int serialNo = in.readIntLE();// 4
        int bodyLen = in.readIntLE();// 4
        byte[] bodySHA1Bytes = new byte[128];
        in.readBytes(bodySHA1Bytes);// 128
        byte compressAlgorithm = in.readByte();// 1
        long reserved = in.readLongLE();// 8

        MessageHeader header = new MessageHeader();
        header.setSzHeaderFlag(szHeaderFlag.toString());
        header.setMsgType(msgType);
        header.setProtoFmtType(protoFmtType);
        header.setProtoVer(protoVer);
        header.setSerialNo(serialNo);
        header.setBodyLen(bodyLen);
        header.setBodySHA1(bodySHA1Bytes);
        header.setCompressAlgorithm(compressAlgorithm);
        header.setReserved(reserved);

        return header;
    }

    /**
     * 写入报文头字节流
     *
     * @param header
     * @param out
     */
    public static void writeHeader(MessageHeader header, ByteBuf out) {
        out.writeCharSequence("HS", StandardCharsets.UTF_8);// 2
        out.writeShortLE(header.getMsgType());// 2
        out.writeByte(header.getProtoFmtType());// 1
        out.writeByte(header.getProtoVer());// 1
        out.writeIntLE(header.getSerialNo());// 4
        out.writeIntLE(header.getBodyLen());// 4
        if (header.getBodySHA1() != null) {
            out.writeBytes(header.getBodySHA1());// 128
        } else {
            out.writeBytes(emptyBodySHA1);
        }
        out.writeByte(header.getCompressAlgorithm());// 1
        out.writeLongLE(header.getReserved());// 8
    }

    /**
     * 解析报文体
     *
     * @param header
     * @param bodyBytes
     * @return
     */
    public static SocketMessage parseBody(MessageHeader header, byte[] bodyBytes) {
        if (header.getProtoFmtType() != 0) {
            logger.error("暂不支持其他协议");
            return null;
        }
        try {
            SocketMessage socketMsg = null;
            MessageLite protobufMsgBody = null;
            if (header.getMsgType() == MsgType.REQUEST) {
                protobufMsgBody = PBRequest.parseFrom(bodyBytes);
                socketMsg = new ProtoBufRequestMessage();
            } else if (header.getMsgType() == MsgType.RESPONSE) {
                protobufMsgBody = PBResponse.parseFrom(bodyBytes);
                socketMsg = new ProtoBufResponseMessage();
            } else if (header.getMsgType() == MsgType.NOTIFY) {
                protobufMsgBody = PBNotify.parseFrom(bodyBytes);
                socketMsg = new ProtoBufNofityMessage();
            } else if (header.getMsgType() == MsgType.HEART_BEAT) {
                protobufMsgBody = HeartBeat.parseFrom(bodyBytes);
                socketMsg = new ProtoBufHeartBeatMessage();
            } else {
                logger.error("不识别的消息类型------msgType:{}", header.getMsgType());
            }
            if (protobufMsgBody == null) {
                logger.error("解码器传入有误---{}", header.getMsgType());
                return null;
            }
            socketMsg.setHeader(header);
            socketMsg.setMsgBody(protobufMsgBody);
            return socketMsg;
        } catch (Exception e) {
            logger.error("Protobuf-解码异常", e);
        }
        return null;
    }

    /**
     * 获取报文体字节数组
     *
     * @param msg
     * @return
     */
    public static byte[] getBodyBytes(SocketMessage msg) {
        MessageHeader header = msg.getHeader();
        byte[] bodyBytes = null;

        if (ProtoFmtType.PROTOBUF != header.getProtoFmtType()) {
            logger.info("暂时不支持非Protobuf类型的其他格式消息");
            return null;
        }

        if (MsgType.REQUEST == header.getMsgType()) {
            PBRequest request = msg.getMsgBody();
            bodyBytes = request.toByteArray();
        } else if (MsgType.NOTIFY == header.getMsgType()) {
            PBNotify notify = msg.getMsgBody();
            bodyBytes = notify.toByteArray();
        } else if (MsgType.RESPONSE == header.getMsgType()) {
            PBResponse response = msg.getMsgBody();
            bodyBytes = response.toByteArray();
        } else if (MsgType.HEART_BEAT == header.getMsgType()) {
            HeartBeat heartBeat = msg.getMsgBody();
            if (heartBeat != null) {
                bodyBytes = heartBeat.toByteArray();
            } else {
                return new byte[0];
            }
        } else {
            logger.warn("不识别的消息类型------msgType:{}", header.getMsgType());
        }

        return bodyBytes;
    }

    /**
     * 验证签名
     *
     * @param header
     * @param bodyBytes
     * @param secretKey
     * @return
     */
    public static boolean signVerify(MessageHeader header, byte[] bodyBytes, String secretKey) {
        if (StringUtils.isBlank(secretKey)) {
            logger.info("签名秘钥为空，不验签");
            return true;
        }

        if (MsgType.REQUEST == header.getMsgType() || MsgType.RESPONSE == header.getMsgType() || MsgType.NOTIFY == header.getMsgType()) {
            if (header.getBodySHA1() == null || header.getBodySHA1().length == 0) {
                logger.warn("Signature empty.");
                return false;
            }

            try {
                if (RsaSignature.verify(bodyBytes, header.getBodySHA1(), secretKey)) {
                    return true;
                } else {
                    logger.warn("Signature not passed.");
                }
            } catch (Exception e) {
                logger.error("Signature verify error", e);
            }
        } else {
            return true;
        }

        return false;
    }

    /**
     * 报文体签名签名
     *
     * @param header
     * @param bodyBytes
     * @param secretKey
     * @return
     */
    public static boolean sign(MessageHeader header, byte[] bodyBytes, String secretKey) {
        if (StringUtils.isBlank(secretKey)) {
            logger.info("签名秘钥为空，不签名");
            return true;
        }

        if (MsgType.REQUEST == header.getMsgType() || MsgType.RESPONSE == header.getMsgType() || MsgType.NOTIFY == header.getMsgType()) {
            try {
                header.setBodySHA1(RsaSignature.sign(bodyBytes, secretKey));
                return true;
            } catch (Exception e) {
                logger.error("Signature error", e);
            }
        } else {
            return true;
        }
        return false;
    }

}
