package com.huasheng.quant.open.gateway.sdk.netty.pipeline.client;

import com.huasheng.quant.open.gateway.sdk.handlers.Application;
import com.huasheng.quant.open.gateway.sdk.netty.pipeline.AbstractSessionHandler;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.constant.ResponseMsgTypeProto.ResponseMsgType;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.HeartBeatProto;
import com.huasheng.quant.open.gateway.sdk.protobuf.common.msg.ResponseProto.PBResponse;
import com.huasheng.quant.open.gateway.sdk.protocol.MessageHeader;
import com.huasheng.quant.open.gateway.sdk.protocol.MsgType;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufHeartBeatMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoFmtType;
import com.huasheng.quant.open.gateway.sdk.protocol.SocketMessage;
import com.huasheng.quant.open.gateway.sdk.protocol.session.Session;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class ClientSessionHandler extends AbstractSessionHandler {

    private static final Logger logger = LoggerFactory.getLogger(ClientSessionHandler.class);

    protected ClientSessionHandler(Application fixApplication) {
        super(fixApplication);
    }

    /**
     * 超时客户端发送心跳请求
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) evt;
            if (event.state() == IdleState.READER_IDLE || event.state() == IdleState.WRITER_IDLE) {
                Session session = getSession(ctx);
                if (session == null || session.getAndIncrementLossConnectCount() >= 3) {
                    // logger.info("Connection timeout, closed: {}", ctx.channel().remoteAddress());
                }

                SocketMessage heartBeatRequest = createHeartBeatRequest();
                ctx.write(heartBeatRequest);
                ctx.flush();
            }
        } else {
            super.userEventTriggered(ctx, evt);
        }
    }

    private SocketMessage createHeartBeatRequest() {
        SocketMessage heartBeatRequest = new ProtoBufHeartBeatMessage();
        HeartBeatProto.HeartBeat heartBeatReqPB = HeartBeatProto.HeartBeat.newBuilder().build();
        heartBeatRequest.setPayload(heartBeatReqPB);
        return heartBeatRequest;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, SocketMessage msg, List<Object> out) throws Exception {
        final MessageHeader header = msg.getHeader();
        Session session = getSession(ctx);
        int msgType = header.getMsgType();
        int protoFmtType = header.getProtoFmtType();
        ResponseMsgType requestMsgType = null;

        if (ProtoFmtType.PROTOBUF == protoFmtType && MsgType.RESPONSE == msgType) {
            PBResponse pbResponse = msg.getMsgBody();
            requestMsgType = pbResponse.getResponseMsgType();
        } else if (ProtoFmtType.PROTOBUF == protoFmtType && MsgType.HEART_BEAT == msgType) {
            // 心跳
            if (session != null) {
                session.resetLossConnectCount();
            }
            return;
        }

        if (requestMsgType == ResponseMsgType.InitConnectResponseMsgType) { //登陆响应消息
            if (session != null) {
                getLogger().info("Fix Session Established.");
                out.add(msg);
            } else {
                throw new IllegalStateException("Duplicate Logon Request. Session Already Established.");
            }
        } else {
            if (session != null) {
                session.resetLossConnectCount();
            }

            if (session == null) {
                getLogger().error("Session not established. Skipping message: {}", msg);
                ctx.channel().close();
                return;
            }
            out.add(msg);
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) {
        getLogger().info("Connection established, starting Client binary session.");
    }

    @Override
    protected Logger getLogger() {
        return logger;
    }
}
