package com.huasheng.quant.open.gateway.sdk.netty.pipeline.client;

import com.huasheng.quant.open.gateway.sdk.handlers.Application;
import com.huasheng.quant.open.gateway.sdk.protocol.ProtoBufRequestMessage;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.PooledByteBufAllocator;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;
import java.net.SocketAddress;

public class SocketClient {

    private static final Logger logger = LoggerFactory.getLogger(SocketClient.class);
    
    private Channel channel;
    private EventLoopGroup bossEventLoopGroup;
    private EventLoopGroup workerEventLoopGroup;
    private Application socketApplication;

    public SocketClient(Application socketApplication) {
        this.socketApplication = socketApplication;
    }
    
    public Channel connect(String host, int port) throws InterruptedException {
        return connect(new InetSocketAddress(host, port));
    }

    public Channel connect(SocketAddress serverAddress) throws InterruptedException {
        final Channel channel = connectAsync(serverAddress).sync().await().channel();
        assert (channel != null) : "Channel must be set";
        return channel;
    }

    public ChannelFuture connectAsync(SocketAddress serverAddress) {
        logger.info("HS API Client is starting");
        final Bootstrap b = new Bootstrap();
        bossEventLoopGroup = new NioEventLoopGroup();
        workerEventLoopGroup = new NioEventLoopGroup(Runtime.getRuntime().availableProcessors());
        b.group(bossEventLoopGroup)
                .channel(NioSocketChannel.class)
                .remoteAddress(serverAddress)
                .option(ChannelOption.TCP_NODELAY,
                        Boolean.parseBoolean(System.getProperty(
                                "nfs.rpc.tcp.nodelay", "true")))
                .option(ChannelOption.ALLOCATOR, new PooledByteBufAllocator())
                .handler(new InitiatorChannelInitializer<SocketChannel>(workerEventLoopGroup, socketApplication))
                .validate();

        final ChannelFuture connectFuture = b.connect();
        ChannelFuture channelFuture = connectFuture.addListener(future -> channel = connectFuture.channel());
        
        logger.info("FixClient connection active status: {}", channelFuture.channel().isActive());
        return channelFuture;
    }

    public ChannelFuture disconnectAsync() {
        logger.info("Closing connection to {}", channel.remoteAddress());
        if (workerEventLoopGroup.isTerminated() && bossEventLoopGroup.isTerminated()) {
            logger.info("Event executor has terminated close channel.");
            return channel.close();
        }
        return channel.close().addListener(future -> {
            if (workerEventLoopGroup != null && !workerEventLoopGroup.isTerminated())
                workerEventLoopGroup.shutdownGracefully();
            if (bossEventLoopGroup != null && !bossEventLoopGroup.isTerminated())
                bossEventLoopGroup.shutdownGracefully();
            bossEventLoopGroup = null;
            workerEventLoopGroup = null;
            logger.info("Connection to {} was closed.", channel.remoteAddress());
        });
    }

    public void disconnect() throws InterruptedException {
        disconnectAsync().await();
    }

    public void send(ProtoBufRequestMessage socketMessage) {
        boolean active = isActive();
        logger.info("Client connection active status before send: {}", active);
        if (active) {
            ChannelFuture future = channel.writeAndFlush(socketMessage);
            future.addListener(new ChannelFutureListener() {
                @Override
                public void operationComplete(ChannelFuture future) throws Exception {
                    logger.info("write {} success.", channel.remoteAddress());
                }
            });
        } else {
            logger.warn("Client connection is inactive status, skip send request: {}", socketMessage.getRequestId());
        } 
    }

    public boolean isActive() {
        return channel != null && channel.isActive();
    }

    public void setSocketApplication(Application application) {
        this.socketApplication = application;
    }
    
}
