package com.huasheng.quant.open.gateway.sdk.utils;

import com.huasheng.quant.open.gateway.sdk.exception.OpenApiRuntimeException;
import okhttp3.ConnectionPool;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.MultipartBody.Builder;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.ResponseBody;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;

public class HttpClientUtil {
    private HttpClientUtil() {
        throw new IllegalStateException("Utility class");
    }

    static {
        CLIENT = new OkHttpClient.Builder()
                .connectionPool(new ConnectionPool(500, 60, TimeUnit.MINUTES))
                .addInterceptor(new HttpInterceptor())
                .retryOnConnectionFailure(false)
                .callTimeout(10, TimeUnit.SECONDS)
                .connectTimeout(3, TimeUnit.SECONDS)
                .hostnameVerifier(new SSLSocketClient().getHostnameVerifier())
                .build();
    }

    private static final OkHttpClient CLIENT;
    private static final Logger LOGGER = LoggerFactory.getLogger(HttpClientUtil.class);
    private static final MediaType JSON_UTF8 = MediaType.parse("application/json; charset=utf-8");

    public static Optional<String> postBody(String url, String body) {
        RequestBody requestBody = RequestBody.create(body, JSON_UTF8);
        return call(url, requestBody, null);
    }

    public static Optional<String> multipartForm(String url, String fileParamName, File file, Map<String, String> params) {
        Builder builder = new Builder()
                .setType(MultipartBody.FORM)
                .addFormDataPart(fileParamName, file.getName(), RequestBody.create(file, MediaType.parse("multipart/form-data")));
        params.forEach(builder::addFormDataPart);
        RequestBody requestBody = builder.build();
        return call(url, requestBody, null);
    }

    public static Optional<String> post(String url, Map<String, String> params) {
        FormBody.Builder formBuilder = new FormBody.Builder();
        if (params != null) {
            for (Entry<String, String> entry : params.entrySet()) {
                formBuilder.add(entry.getKey(), entry.getValue());
            }
        }
        RequestBody requestBody = formBuilder.build();
        return call(url, requestBody, null);
    }

    public static Optional<String> get(String url, String name, String value) {
        url += "?" + name + "=" + value;
        return call(url, null, null);
    }

    public static Optional<String> get(String url, Map<String, String> params) {
        url = url + "?" + HttpClientUtil.buildQuery(params);
        return call(url, null, null);
    }

    private static String buildQuery(Map<String, String> params) {
        if (params == null || params.isEmpty()) {
            return "";
        }
        StringBuilder query = new StringBuilder();
        Set<Entry<String, String>> entries = params.entrySet();
        boolean hasParam = false;
        for (Entry<String, String> entry : entries) {
            String name = entry.getKey();
            String value = entry.getValue();
            // 忽略参数名或参数值为空的参数
            if (!StringUtils.isAnyBlank(name, value)) {
                if (hasParam) {
                    query.append("&");
                } else {
                    hasParam = true;
                }
                try {
                    query.append(name).append("=").append(URLEncoder.encode(value, "UTF-8"));
                } catch (UnsupportedEncodingException neverHappen) {
                    LOGGER.info("UTF-8编码错误", neverHappen);
                }
            }
        }
        return query.toString();
    }

    public static byte[] download(String url) throws IOException {
        LOGGER.info("下载文件链接：{}", url);
        Request.Builder requestBuilder = new Request.Builder().url(url);
        Request request = requestBuilder.build();
        Response response = CLIENT.newCall(request).execute();
        ResponseBody responseBody = response.body();
        if (responseBody == null) {
            throw new IOException(url + "下载文件出错");
        }
        return responseBody.bytes();
    }

    private static Optional<String> call(String url, RequestBody requestBody, Map<String, String> headers) {
        try {
            ResponseBody responseBody = newCall(url, requestBody, headers);
            if (responseBody == null) {
                return Optional.empty();
            }
            String bodyText = responseBody.string();
            return Optional.of(bodyText);
        } catch (IOException e) {
            throw new OpenApiRuntimeException("请求接口 [" + url + "] 发生异常: " + e.getMessage(), e);
        }
    }

    private static ResponseBody newCall(String url, RequestBody requestBody, Map<String, String> headers) throws IOException {
        Request.Builder requestBuilder;
        if (requestBody != null) {
            requestBuilder = new Request.Builder().url(url).post(requestBody);
        } else {
            requestBuilder = new Request.Builder().url(url);
        }
        if (headers != null) {
            headers.forEach(requestBuilder::header);
        }
        Request request = requestBuilder.build();
        Response response = CLIENT.newCall(request).execute();
        return response.body();
    }
}
